
/*
xpvox copyright (c) 2005 Jan Panteltje

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/



#include "forms.h"
#include "xpvox.h"

extern FD_xpvox *fdui;



int print_usage()
{
fprintf(stderr, "\
Usage:               xpvox [-h] [-o] [-v]\n\n\
-c                   use the cmix mixer program to set mic 100%%, igain 50%%, vol 100%%, and igain as source.\n\
-h                   help (this help).\n\
-i                   no I/O, this allows use as non root if you only want to make voice controled recordings,\n\
                     serial port and RTS are not accessed.\n\
-l filename          log actual transmit times (RTS on) and possible audio files transmitted to filename.\n\
-o                   send PCM as 44100 16 bits mono stream to stdout, use with lame to make mp3.\n\
-s                   output no silence to stdout.\n\
-v                   debug (prints functions and arguments).\n\
example:\n\
xpvox -o | lame -r -s 44100 -m m -x - out.mp3\n\
");

return 1;
} /* end function print_usage */


int main(int argc, char *argv[])
{
FD_xpvox *fd_xpvox;
FL_IOPT fl_iopt;
char *ptr;
int a;
Pixmap p, mask;
char temp[1024];
unsigned w, h;
FILE *fptr;

/* get user info */
userinfo = getpwuid(getuid() );

/* get home directory */ 
home_dir = (char *)strsave(userinfo -> pw_dir);
if(! home_dir)
	{
	fprintf(stderr, "xpvox: main(): cannot allocate space for home_dir, aborting.\n");
	exit(1);
	}

user_name = (char *)strsave(userinfo -> pw_name);
if(! user_name)
	{
	fprintf(stderr, "xpvox: main(): cannot allocate space for user_name, aborting.\n");
	exit(1);
	}

/* against flashing */
/* set double buffering */
fl_iopt.doubleBuffer = 1;
fl_set_defaults(FL_PDDouble, &fl_iopt);
                   
fl_initialize(&argc, argv, 0, 0, 0);

fd_xpvox = create_form_xpvox();

/* set the icon */
sprintf(temp, "%s/.xpvox/xpvox.xpm", home_dir);
fptr = fopen(temp, "r");
if(fptr)
	{
	fclose(fptr);

	p = fl_read_pixmapfile(fl_root, temp, &w, &h, &mask, 0, 0, 0);

	fl_set_form_icon(fdui -> xpvox, p, mask);                 
	}
else
	{
	fprintf(stderr, "no icon %s found\n", temp);
	}

/* fill-in form initialization code */

/* defaults */
debug_flag = 0;
output_flag = 0;
transmission_inhibit_flag = 0;

dsp_speed = DEFAULT_DSP_SPEED;
dsp_stereo = 0; // mono
samplesize = 16;

threshold = DEFAULT_THRESHOLD;
on_milli_seconds = DEFAULT_ON_TIME;
off_milli_seconds = DEFAULT_OFF_TIME;
beacon_period_time = DEFAULT_BEACON_PERIOD_TIME;

set_mixer_flag = 0;

selected_sound = 0;

serial_device = strsave(DEFAULT_SERIAL_PORT);
if(! serial_device)
	{
	fprintf(stderr, "xpvox: could not allocate space for serial device, aborting.\n");

	exit(1);
	}

audio_input_device_number = DEFAULT_AUDIO_INPUT_DEVICE_NUMBER;
set_audio_input_device(audio_input_device_number);

audio_output_device_number = DEFAULT_AUDIO_OUTPUT_DEVICE_NUMBER;
set_audio_output_device(audio_output_device_number);

auto_play_sequence_number = 1;
output_no_silence_flag = 0;
no_io_flag = 0;
show_sounds_without_path_flag = 0;
log_flag = 0;
old_rts_is_on_flag = -1;
agc_init_flag = 0;
agc_level = 0.7;
agc_multiplier = 1.0;
use_xpequ_flag = 0;

/* end defaults */

ptr = (char *) load_setting("log_filename");
if( ! ptr)
	{
	/* report about usings defaults in load settings() */
	}
else
	{
	sscanf(ptr, "%as", &log_filename);
	}

/* set all time vars */
ogettime(weekday, month, &monthday, &hour, &minute, &second, &year);

load_audio_settings();

while(1)
	{
	a = getopt(argc, argv, "chil:ovs");
	if(a == -1) break;

	switch(a)
		{
		case 'c': // use cmix
			set_mixer_flag = 1;
			break;
		case 'h': // help
			print_usage();
			exit(1);
			break;
		case 'i': // no io mode, serial port not accessed.
			no_io_flag = 1;
			break;
		case 'l': // log transmissions and possible audio files used
			if(log_filename) free(log_filename);
			log_filename = strsave(optarg);
			if(! log_filename)
				{
				fprintf(stderr, "xpvox: main(): could not allocate space for log_filename, aborting.\n");
				exit(1);
				}

			log_flag = 1;
			break;
		case 'o': // PCM to stdout
			output_flag = 1;
			break;
		case 's': // output no silence to stdout
			output_no_silence_flag = 1;
			break;
		case 'v': // debug
			debug_flag = 1;
			break;
        case -1:
        	break;
		case '?':
			if (isprint(optopt) )
 				{
 				fprintf(stderr, "Unknown option `-%c'.\n", optopt);
 				}
			else
				{
				fprintf(stderr, "Unknown option character `\\x%x'.\n",\
				optopt);
				}
			print_usage();

			exit(1);
			break;			
		default:
			print_usage();

			exit(1);
			break;
		}/* end switch a */
	}/* end while getopt() */

/* for direct io to serial port */
if(! no_io_flag) iopl(3);

#ifdef TIMING_CHECKS
start_timeval = malloc(sizeof(struct timeval) );
if(! start_timeval)
	{
	fprintf(sterr, "could not allocate space for start_timeval\n");

	exit(1);
	}

current_timeval = malloc(sizeof(struct timeval) );
if(! current_timeval)
	{
	fprintf(stderr, "could not allocate space for current_timeval\n");

	exit(1);
	}
#endif // TIMING_CHECKS

/* get sound dir */
sound_dir = strsave(home_dir);
if(! sound_dir)
	{
	fprintf(stderr, "xpvox: main cannot not allocate space for sound_dir, aborting.\n");

	exit(1);
	}

ptr = (char *) load_setting("sound_dir");
if( ! ptr)
	{
	/* report about usings defaults in load settings() */
	}
else
	{
	sscanf(ptr, "%as", &sound_dir);
	}

if(debug_flag)
	{
	fprintf(stderr, "sound_dir=%s ptr=%s\n", sound_dir, ptr);
	}

/* tx off */
if(! set_rts(serial_device, 0) )
	{
	fprintf(stderr, "xpvox: main(): set_rts() to zero failed, aborting.\n");

	exit(1);
	}

if(! start_audio() )
	{
	fprintf(stderr, "xpvox: main(): could not initilize audio thread, try a different dsp input device.\n");

	fl_show_alert(\
	"could not initialize audio thread", audio_input_device, "try a different input device", 0);

	/* allow user to select an other dsp input device */
	}

minimum_db = 20 * log10(1.0 / ( (1 << 15) / 2) );
if(debug_flag)
	{
	fprintf(stderr, "minimum_db=%.2f\n", minimum_db);
	}

fl_set_slider_bounds(fd_xpvox -> threshold_slider, 0, minimum_db);
fl_set_slider_bounds(fd_xpvox -> audio_peak_indicator_slider, 0, minimum_db);

ptr = (char *) load_setting("threshold");
if( ! ptr)
	{
	/* report about usings defaults in load settings() */
	}
else
	{
	sscanf(ptr, "%lf", &threshold);
	}

if(debug_flag)
	{
	fprintf(stderr, "threshold=%.2f ptr=%s\n", threshold, ptr);
	}

fl_set_slider_value(fd_xpvox -> threshold_slider, threshold);

milli_seconds_per_sample = 1000.0 / (double)dsp_speed;

if(debug_flag)
	{
	fprintf(stderr, "milli_seconds_per_sample=%.2f\n", milli_seconds_per_sample);
	fprintf(stderr, "on_milli_seconds=%.2f off_milli_seconds=%.2f\n", on_milli_seconds, off_milli_seconds);
	fprintf(stderr, "high_samples=%d low_samples=%d\n", high_samples, low_samples);
	}

ptr = (char *) load_setting("on_milli_seconds");
if( ! ptr)
	{
	/* report about usings defaults in load settings() */
	}
else
	{
	sscanf(ptr, "%lf", &on_milli_seconds);
	}

if(debug_flag)
	{
	fprintf(stderr, "on_milli_seconds=%.2f ptr=%s\n", on_milli_seconds, ptr);
	}

fl_set_slider_value(fd_xpvox -> on_time_slider, on_milli_seconds);


ptr = (char *) load_setting("off_milli_seconds");
if( ! ptr)
	{
	/* report about usings defaults in load settings() */
	}
else
	{
	sscanf(ptr, "%lf", &off_milli_seconds);
	}

if(debug_flag)
	{
	fprintf(stderr, "off_milli_seconds=%.2f ptr=%s\n", on_milli_seconds, ptr);
	}

fl_set_slider_value(fd_xpvox -> off_time_slider, off_milli_seconds);

ptr = (char *)load_setting("selected_sound");
if(! ptr)
	{
	/* report about usings defaults in load settings() */
	}
else
	{
	sscanf(ptr, "%d", &selected_sound);
	}

if(debug_flag)
	{
	fprintf(stderr, "selected_sound=%d\n", selected_sound);
	}

load_sounds();
show_sounds(SHOW_SELECTED);

//(int)high_samples = (double) on_milli_seconds / (double) milli_seconds_per_sample;
high_samples = on_milli_seconds / milli_seconds_per_sample;

//(int)low_samples = (double) off_milli_seconds / (double) milli_seconds_per_sample;
low_samples = off_milli_seconds / milli_seconds_per_sample;

/* show the first form */
fl_show_form(fd_xpvox -> xpvox, FL_PLACE_FREE, FL_FULLBORDER, PROG_VERSION);

fl_do_forms();

return 0;
} /* end function main */


